#ifndef HUHFA_H
#define HUHFA_H

#include <fstream>
#include <iostream>
#include <sstream>
#include <vector>
#include <map>
#include <set>

using namespace std;

struct classcomp
{
    bool operator() (const vector<int> a, const vector<int> b) const;
};

bool classcomp::operator() (const vector<int> a, const vector<int> b) const
    // orders inequalities lexicographic on their manhattan metric
{
    if(a.size()!=b.size()){
        return a.size()<b.size();
    }
    else{
        int h=0;
        for(int i=0; i<a.size(); i++){
            if(a[i]<0){   h+=a[i];}
            else{           h-=a[i];}
            if(b[i]<0){    h-=b[i];}
            else{           h+=b[i];}
        }
        if(h!=0){
            return (h>0);
        }
        else{
            for(int i=0; i<a.size(); i++){
                if(a[i]!=b[i]){
                    return a[i]<b[i];
                }
            }
            return false;
        }
    }
}



class huhfa{
    //raw data of polytope
        int n; //Number of Variables
        int f; //Number of Facets
        int v; //Number of Vertices
        int p; //Number of Permutations
        bool input_fail;
        vector<string> variables;
        vector<vector<int> > vertices;
        vector<vector<int> > inequalities;
        set<vector<int>, classcomp> sort_ineq;
        vector<vector<vector<pair<int, int> > > > affinemaps;
    //synthesized data of polytope
        vector<pair<int,int> > Facet_to_Class;
        vector<vector<bool> > Index_to_Incidence;
        map<vector<bool>,int> Incidence_to_Index;
        vector<vector<int> > Permutations_Table;
        map<int ,map<int, vector<int> > >  classes;
    //methods
    public:
        huhfa(string source);
        void synthesize();
        void classify(bool complete);
        void output(string result, bool rep);

        void Check_Index_to_Incidence(){
            for(int i=0; i<f; i++){
                for(int j=0; j<v; j++){
                    cout<<Index_to_Incidence[i][j]<<" ";
                }
                cout<<endl;
            }
        }

         void Check_Incidence_to_Index(){
            for(map<vector<bool>,int>::iterator it=Incidence_to_Index.begin(); it!=Incidence_to_Index.end(); it++){
                cout<<(*it).second<<endl;
            }
        }

        void Check_Permutation_Table(){
            for(int i=0; i<p; i++){
                for(int j=0; j<v; j++){
                    cout<<Permutations_Table[i][j]<<" ";
                }
                cout<<endl;
            }
        }

        void Check_classes(){
            for(map<int ,map<int, vector<int> > >::iterator it=classes.begin(); it!=classes.end(); it++){
                for(map<int, vector<int> >::iterator jt=(*it).second.begin(); jt!=(*it).second.end(); jt++){
                    for(int i=0; i<(*jt).second.size(); i++){
                        cout<<(*jt).second[i]<<" ";
                    }
                }
                cout<<endl;
            }
        }




};

#endif

