#include "huhfa.h"

void huhfa::synthesize(){
    if(!input_fail){
        //Copy the Inequalities for later use:
        for(set<vector<int>, classcomp>::iterator it=sort_ineq.begin(); it!=sort_ineq.end(); it++){
            inequalities.push_back(*it);
        }
        //Compute "Index_to_Incidence" and "Incidence_to_Index"
        vector<bool> Incidence;
        int lhs;
        //For every Facet
        for(int i=0; i<f; i++){
            Incidence.clear();
            //Compute the Incidence Vector
            for(int j=0; j<v; j++){
                lhs=0;
                for(int k=0; k<n; k++){
                    lhs+=inequalities[i][k]*vertices[j][k];
                }
                Incidence.push_back(lhs==inequalities[i][n]);
            }
            //Store the maps
            Index_to_Incidence.push_back(Incidence);
            if(Incidence_to_Index.find(Incidence)==Incidence_to_Index.end()){
                Incidence_to_Index[Incidence]=i;
            }
        }
        if(Index_to_Incidence.size()>Incidence_to_Index.size() ){
            cerr<<"Warning: There are "<<Index_to_Incidence.size()-Incidence_to_Index.size()<<" redundant Inequalities."<<endl<<endl;
        }

        //Compute map for the vertices
        map<vector<int>, int> Vertex_to_Index;
        for(int i=0; i<v; i++){
            Vertex_to_Index[vertices[i]]=i;
        }

        //Compute "Permutation_Table"
        vector<int> permutation;
        vector<int> Image;
        //For every affine map
        for(int i=0; i<p; i++){
            permutation.clear();
            bool error=false;
            //For every Vertex
            for(int j=0; j<v; j++){
                Image.clear();
                //Compute the Image of the Vertex under the affine map

                for(int k=0; k<n; k++){
                    lhs=0;
                    for(int l=0; l<affinemaps[i][k].size(); l++){
                        if(affinemaps[i][k][l].first!=-1){
                            lhs+=vertices[j][affinemaps[i][k][l].first]*affinemaps[i][k][l].second;
                        }
                        else{
                            lhs+=affinemaps[i][k][l].second;
                        }
                    }
                    Image.push_back(lhs);
                }
                //Store the Index permutation of the vertices
                if(Vertex_to_Index.find(Image)!=Vertex_to_Index.end()){
                    permutation.push_back(Vertex_to_Index[Image]);
                }
                else{
                    error=true;
                }
            }
            //Check if the map really is a permutation of vertices
            if(!error){
                Incidence.clear();
                for(int j=0; j<v; j++){
                    Incidence.push_back(false);
                }
                for(int j=0; j<v; j++){
                    Incidence[permutation[j]]=true;
                }
                for(int j=0; j<v; j++){
                    error=!(!error && Incidence[j]);
                }
            }
            if(!error){
                Permutations_Table.push_back(permutation);
            }
            else{
                cerr<<"Error: "<<i+1<<" is not a symmetry."<<endl;
                input_fail=true;
            }
        }
    }
}


